<?php

namespace Service;

class Order {

	protected $entityManager;
	protected $util;
	protected $tableService;
	protected $billService;
	protected $impressaoPedido;
	protected $parametrosAPI;

	public function __construct(
		\Doctrine\ORM\EntityManager $entityManager,
		\Util\Util $util,
		\Service\Table $tableService,
		\Service\Bill $billService,
		\Service\Impressao $impressaoService,
		\Service\KDS $KDS,
		\Odhen\API\Service\impressaoPedido $impressaoPedido,
		\Odhen\API\Service\Parametros $parametrosAPI
	) {
		$this->entityManager = $entityManager;
		$this->util = $util;
		$this->tableService = $tableService;
		$this->billService = $billService;
		$this->impressaoService = $impressaoService;
		$this->KDS = $KDS;
		$this->impressaoPedido = $impressaoPedido;
		$this->parametrosAPI = $parametrosAPI;
	}

	private function setDefaultReasonValues($motivo) {
		if (empty($motivo['CDGRPOCOR'])) {
			$motivo['CDGRPOCOR'] = null;
		}
		if (empty($motivo['CDOCORR'])) {
			$motivo['CDOCORR'] = null;
		}
		if (empty($motivo['TXPRODCOMVEN'])) {
			$motivo['TXPRODCOMVEN'] = null;
		}
		return $motivo;
	}

	public function cancelaProduto($dataset){ //Detaset Params: chave, mode, dscomanda, produto, motivo, supervisor
		$session = $this->util->getSessionVars($dataset['chave']);
		$NRCOMANDA = $dataset['NRCOMANDA'];
		$NRVENDAREST = $dataset['NRVENDAREST'];
		$motivo = self::setDefaultReasonValues($dataset['motivo']);
		$stAmbiente = null;
		$IDPRODPRODUZ = $dataset['IDPRODPRODUZ'];
		// Buscando a mesa.
		$valMesaOrigem = $this->tableService->dadosMesa($session['CDFILIAL'], $session['CDLOJA'], $NRCOMANDA, $NRVENDAREST, false);
		$mesaOrigem = $valMesaOrigem['NRMESA'];

        /* CHECK FOR ITEMS IN MOVCAIXADLV */
        $params = array(
            'CDFILIAL' => $session['CDFILIAL'],
            'NRVENDAREST' => $NRVENDAREST
        );
        $valTrans = $this->entityManager->getConnection()->fetchAll("SQL_BUSCA_TRANSACOES_MOVCAIXADLV", $params);
        if (!empty($valTrans)){
            return array('funcao' => '0', 'error' => '462');
        }

		if ($dataset['mode'] == 'C'){
			// valida e busca dados da comanda
			$valComanda = $this->billService->dadosComanda($session['CDFILIAL'], $NRCOMANDA, $NRVENDAREST, $session['CDLOJA']);
			$stDsComanda = $valComanda['DSCOMANDA'];
			$stNrMesa = $valComanda['NRMESA'];
		}
		else if ($dataset['mode'] == 'M'){
			// valida e busca os dados da mesa
			$valMesa = $this->tableService->dadosMesa($session['CDFILIAL'], $session['CDLOJA'], $NRCOMANDA, $NRVENDAREST);
			$stDsComanda = '';
			$stNrMesa = $valMesa['NRMESA'];
			$params = array($session['CDFILIAL'], $session['CDLOJA'], $stNrMesa);
			$stAmbiente = $this->entityManager->getConnection()->fetchAssoc("SQL_AMBIENTE", $params);
			if (!empty($stAmbiente)) $stAmbiente = $stAmbiente['NMSALA'];

		}

		$stNrVendaRest = $dataset['produto']['NRVENDAREST'];
		$stNrComanda = $dataset['produto']['nrcomanda'];

		// ---

        $FILIALVIGENCIA = $session['FILIALVIGENCIA'];
		$NRCONFTELA   = $session['NRCONFTELA'];
        $DTINIVIGENCIA = new \DateTime($session['DTINIVIGENCIA']);
		$CDFILIAL     = $session['CDFILIAL'];
		$NRPRODCOMVEN = $dataset['produto']['NRPRODCOMVEN'];
		$params = array(
            'FILIALVIGENCIA' => $FILIALVIGENCIA,
            'NRCONFTELA' => $NRCONFTELA,
            'DTINIVIGENCIA' => $DTINIVIGENCIA,
            'CDFILIAL' => $CDFILIAL,
            'NRCOMANDA' => $stNrComanda,
            'NRVENDAREST' => $stNrVendaRest,
            'NRPRODCOMVEN' => $NRPRODCOMVEN
        );
        $types = array(
            'DTINIVIGENCIA' => \Doctrine\DBAL\Types\Type::DATETIME
        );
		$itemSelecionado = $this->entityManager->getConnection()->fetchAssoc("SQL_SELECIONA_ITEM", $params, $types);

		/*$produtosCampanha = $this->parametrosAPI->buscaCampanhaCompreGanhe($CDFILIAL);
		
		$produtosCampanhaCancela = array();
		array_push($produtosCampanhaCancela, $itemSelecionado);

		foreach ($produtosCampanha as $prod){
			if($prod["CDPRODUTO"] == $itemSelecionado["CDPRODUTO"]){
				foreach($prod["products"] as $p){
					$params = array(
            		'FILIALVIGENCIA' => $FILIALVIGENCIA,
            		'NRCONFTELA' => $NRCONFTELA,
            		'DTINIVIGENCIA' => $DTINIVIGENCIA,
            		'CDFILIAL' => $CDFILIAL,
            		'NRCOMANDA' => $stNrComanda,
            		'NRVENDAREST' => $stNrVendaRest,
            		'CDPRODUTO' => $p["CDPRODUTO"]
        		);
        		$types = array('DTINIVIGENCIA' => \Doctrine\DBAL\Types\Type::DATETIME);
				$itemSelecionado = $this->entityManager->getConnection()->fetchAssoc("SQL_SELECIONA_ITEM_CAMPANHA", $params, $types);
				if($itemSelecionado != null) array_push($produtosCampanhaCancela, $itemSelecionado);
				}
				
			}
		}

		if(!empty($produtosCampanha)){
			$acrescimo = $produtosCampanhaCancela[0]['VRACRCOMVEN'];
		}*/
		
        //Seguindo o FOS não é necessário deletar integralmente e sim somente a parcela selecionada
        // Se item tiver sido dividido, deleta o item (integral)
        /*if (!empty($itemSelecionado)) {
            if ($itemSelecionado["IDDIVIDECONTA"] == "S") {
            	$NRCOMANDAORI = $itemSelecionado["NRCOMANDAORI"];
                $NRPRODCOMVEN = $itemSelecionado["NRPRODORIG"];
                $params = array(
                    'FILIALVIGENCIA' => $FILIALVIGENCIA,
                    'NRCONFTELA' => $NRCONFTELA,
                    'DTINIVIGENCIA' => $DTINIVIGENCIA,
                    'CDFILIAL' => $CDFILIAL,
                    'NRCOMANDA' => $NRCOMANDAORI
                );
                $types = array(
                    'DTINIVIGENCIA' => \Doctrine\DBAL\Types\Type::DATETIME
                );
                $item = $this->entityManager->getConnection()->fetchAll("SQL_ITENS_ORIGINAIS", $params, $types);
                
                $codigo = $dataset['produto']['codigo'];
                $QTCANCEL = $dataset['produto']['QTCANCEL'];
                $dataset['produto'] = $item[0];
                $dataset['produto']['codigo'] = $codigo;
                $dataset['produto']['QTCANCEL'] = 0; // Produtos dividos são cancelados integralmente.
                $dataset['produto']['quantidade'] = $item[0]["QTPRODCOMVEN"];
                $data = array(
                    "chave"        => $dataset['chave'],
                    "NRVENDAREST"  => array($dataset['produto']['NRVENDAREST']),
                    "NRCOMANDA"    => array($dataset['produto']['NRCOMANDA']),
                    "selection"    => array($dataset['produto']['NRPRODCOMVEN']),
                    "IDPRODPRODUZ" => $IDPRODPRODUZ,
                    "NRPRODCOMVEN" => $itemSelecionado["NRPRODCOMVEN"]
                );
                $answer = $this->cancelaProdutosDivididos($data, true);
            }
        }*/

        // ---

        // QTCANCEL - quantidade a ser cancelada.
        $dataset['produto']['quantidade'] = $dataset['produto']['QTCANCEL'];

		// verifica se a quantidade é menor que um e aceita somente para produtos pesáveis
		$getProdPes = $this->entityManager->getConnection()->fetchAll("SQL_GET_PROD_PES", array($dataset['produto']['codigo']));

		$produto = $dataset['produto'];

		$comboProd = $produto['NRSEQPRODCOM'] != null && $produto['CDPRODPROMOCAO'] == null;
		if ($comboProd){
    		$params = array(
        		'CDFILIAL' => $CDFILIAL,
        		'NRVENDAREST' => $produto['NRVENDAREST'],
        		'NRCOMANDA' => $produto['nrcomanda'],
        		'NRPRODCOMVEN' => $produto['NRPRODCOMVEN']
    		);
    		$produtosEST = $this->entityManager->getConnection()->fetchAll("SELECT_ITCOMANDAEST_SPECIFIC", $params);
    		if (!empty($produtosEST)) $comboProd = false;
		}

		if ($dataset['produto']['quantidade'] < 1 && $getProdPes[0]['IDPESAPROD'] == 'N' && $comboProd == false && $itemSelecionado["IDDIVIDECONTA"] == 'N'){
			return array('funcao' => '0', 'error' => '051'); //051 - Erro de execução na função.
		}

		// busco os produtos combinados ao produto passado para cancelamento
		$params = array($session['CDFILIAL'],
			$stNrComanda,
			$stNrVendaRest,
			$dataset['produto']['NRSEQPRODCOM']
		);
		$itensCombinados = $this->entityManager->getConnection()->fetchAll("SQL_ITENS_COMBINADOS", $params);

		if(!empty($itensCombinados)){
			$acrescimo = $itensCombinados[0]['VRACRCOMVEN'];
		}

		// busco os dados do produto passado para cancelamento
		$params = array($session['CDFILIAL'], $stNrComanda, $stNrVendaRest, $dataset['produto']['NRPRODCOMVEN']);
		$item = $this->entityManager->getConnection()->fetchAll("SQL_ITEM", $params);

		if (!empty($itemSelecionado)){
			if (($itemSelecionado["IDDIVIDECONTA"] == "S") && (empty($item))) {
				return array('funcao' => '0', 'error' => '454'); // 454 - Já houve o pagemento parcial do item
			}
		}

		if (empty($item)){
			return array('funcao' => '0', 'error' => '039'); //039 - Produto não cadastrado.
		}

		// CANCELAMENTO PARCIAL DE PEDIDOS AGRUPADOS (somente produtos normais).
		if ($dataset['produto']['quantidade'] < floatval($item[0]['QTPRODCOMVEN']) && $dataset['produto']['quantidade'] > 0 && empty($itensCombinados)){
			try {
				/* Open connection and begin transaction. */
				$connection = $this->entityManager->getConnection();
				$connection->beginTransaction();

				$descAtual = isset($dataset['produto']['desconto']) ? $dataset['produto']['desconto'] : 0;

				if(floatval($descAtual) > 0){
					$desconto = $descAtual / floatval($item[0]['QTPRODCOMVEN']);
					$descAtual = $desconto * (floatval($item[0]['QTPRODCOMVEN']) - floatval($dataset['produto']['quantidade']));
				}

				$params = array(
					'QTPRODCOMVEN' => floatval($item[0]['QTPRODCOMVEN']) - floatval($dataset['produto']['quantidade']),
					'CDFILIAL' => $session['CDFILIAL'],
					'NRVENDAREST' => $stNrVendaRest,
					'NRCOMANDA' => $stNrComanda,
					'NRPRODCOMVEN' => $dataset['produto']['NRPRODCOMVEN'],
					'VRDESCCOMVEN' => $descAtual
				);
				$this->entityManager->getConnection()->executeQuery("SQL_ALTERA_QUANTIDADE", $params);

				//updata o pedido no controle de produção (kds)
				$operationParamsArray = array(
					'CDFILIAL'     => $session['CDFILIAL'],
					'NRVENDAREST'  => $stNrVendaRest,
					'NRCOMANDA'    => $stNrComanda,
					'NRPRODCOMVEN' => $dataset['produto']['NRPRODCOMVEN'],
					'QTPRODPEFOS'  => floatval($item[0]['QTPRODCOMVEN']) - floatval($dataset['produto']['quantidade'])
				);
				$this->KDS->insertKDSOPERACAOTEMP($operationParamsArray, 'cancelItem', $session['NRORG']);
				// $this->entityManager->getConnection()->executeQuery("SQL_CANC_QUANT_KDS", $params);

				if (!empty($item[0]['NRCOMANDAORI']) && !empty($item[0]['NRPRODCOMORI'])){
					$params = array(
						'QTPRODCOMVEN' => floatval($item[0]['QTPRODCOMVEN']) - floatval($dataset['produto']['quantidade']),
						'CDFILIAL' => $session['CDFILIAL'],
						'NRVENDAREST' => $dataset['produto']['NRVENDAREST'],
						'NRCOMANDA' => $item[0]['NRCOMANDAORI'],
						'NRPRODCOMVEN' => $dataset['produto']['NRPRODCOMVEN'],
						'VRDESCCOMVEN' => $descAtual
					);
					$this->entityManager->getConnection()->executeQuery("SQL_ALTERA_QUANTIDADE", $params);
				}

				/* Altera a ITHRPEDIDO. */
				// Esta função está servindo apenas como ajuste.
				// Na data atual (06/07/2017) o Waiter/Fast Pass não realiza cancelamento de quantidades específicas.
				// Favor revisar caso seja alterado.
				$params = array(
					floatval($item[0]['QTPRODCOMVEN']) - floatval($dataset['produto']['quantidade']),
					$session['CDFILIAL'],
					substr($dataset['produto']['NRPRODCOMVEN'], strlen($dataset['produto']['NRPRODCOMVEN'])-3),
					$dataset['produto']['NRVENDAREST']
				);


				$connection->commit();
			}
			catch (\Exception $e){
				if($connection != null){
					   $connection->rollback();
				}
				  throw new \Exception($e->getMessage(), 1);
			}
		}
		// CANCELAMENTO DE TODAS AS UNIDADES DO PEDIDO (e produtos com composição).
		else if (($dataset['produto']['quantidade'] == floatval($item[0]['QTPRODCOMVEN']) && $dataset['produto']['quantidade'] > 0) || !empty($itensCombinados)){

			try {
				/* Open connection and begin transaction. */
				$connection = $this->entityManager->getConnection();
				$connection->beginTransaction();

				// o produto a ser eliminado é uma promoção inteligente
				if (!empty($itensCombinados)){
					foreach($itensCombinados as $produto){
						$params = array(
							'TXPRODCOMVEN'   => $motivo['TXPRODCOMVEN'],
							'CDGRPOCOR'      => $motivo['CDGRPOCOR'],
							'CDOCORR'        => $motivo['CDOCORR'],
							'IDSTPRCOMVEN'   => '6',
							'CDSUPERVISOR'   => $dataset['CDSUPERVISOR'],
							'CDFILIAL'       => $session['CDFILIAL'],
							'NRCOMANDA'      => $stNrComanda,
							'NRVENDAREST'    => $stNrVendaRest,
							'NRPRODCOMVEN'   => $produto['NRPRODCOMVEN'],
							'IDPRODPRODUZ'   => $IDPRODPRODUZ,
							'CDOPERCANC'     => $session['CDOPERADOR'],
							'DTHRPRODCANVEN' => $dataset['DTHRPRODCANVEN']
						);
						$types = array(
							'DTHRPRODCANVEN' => \Doctrine\DBAL\TypeS\Type::DATETIME
						);
						$this->entityManager->getConnection()->executeQuery("SQL_CANCELA_ITEM", $params, $types);

						$operationParamsArray = array(
							'CDFILIAL'     => $session['CDFILIAL'],
							'NRVENDAREST'  => $stNrVendaRest,
							'NRCOMANDA'    => $stNrComanda,
							'NRPRODCOMVEN' => $produto['NRPRODCOMVEN']
						);
						$this->KDS->insertKDSOPERACAOTEMP($operationParamsArray, 'cancelItem', $session['NRORG']);

						if (!empty($produto['NRCOMANDAORI']) && !empty($produto['NRPRODCOMORI'])){
							$params = array(
								'TXPRODCOMVEN'   => $motivo['TXPRODCOMVEN'],
								'CDGRPOCOR'      => $motivo['CDGRPOCOR'],
								'CDOCORR'        => $motivo['CDOCORR'],
								'IDSTPRCOMVEN'   => '6',
								'CDSUPERVISOR'   => $dataset['CDSUPERVISOR'],
								'CDFILIAL'       => $session['CDFILIAL'],
								'NRCOMANDA'      => $produto['NRCOMANDAORI'],
								'NRVENDAREST'    => $stNrVendaRest,
								'NRPRODCOMVEN'   => $produto['NRPRODCOMVEN'],
								'IDPRODPRODUZ' 	 => $IDPRODPRODUZ,
								'CDOPERCANC'     => $session['CDOPERADOR'],
								'DTHRPRODCANVEN' => $dataset['DTHRPRODCANVEN']
							);
							$types = array(
								'DTHRPRODCANVEN' => \Doctrine\DBAL\TypeS\Type::DATETIME
							);
							$this->entityManager->getConnection()->executeQuery("SQL_CANCELA_ITEM", $params, $types);
						}
					}
					$params = array($session['CDFILIAL'], $dataset['produto']['nrcomanda'], $dataset['produto']['NRVENDAREST']);
					$acrComVen = $this->entityManager->getConnection()->fetchAll("SQL_ACR_COM_VEN", $params);
					if (!empty($acrComVen)) $acrComVen[0]['VRACRCOMANDA'] -= $acrescimo;
				}/* else if(!empty($produtosCampanhaCancela)){
					foreach($produtosCampanhaCancela as $produto){
						$params = array(
							'TXPRODCOMVEN'   => $motivo['TXPRODCOMVEN'],
							'CDGRPOCOR'      => $motivo['CDGRPOCOR'],
							'CDOCORR'        => $motivo['CDOCORR'],
							'IDSTPRCOMVEN'   => '6',
							'CDSUPERVISOR'   => $dataset['CDSUPERVISOR'],
							'CDFILIAL'       => $session['CDFILIAL'],
							'NRCOMANDA'      => $stNrComanda,
							'NRVENDAREST'    => $stNrVendaRest,
							'NRPRODCOMVEN'   => $produto['NRPRODCOMVEN'],
							'IDPRODPRODUZ'   => $IDPRODPRODUZ,
							'CDOPERCANC'     => $session['CDOPERADOR'],
							'DTHRPRODCANVEN' => $dataset['DTHRPRODCANVEN']
						);
						$types = array(
							'DTHRPRODCANVEN' => \Doctrine\DBAL\TypeS\Type::DATETIME
						);
						$this->entityManager->getConnection()->executeQuery("SQL_CANCELA_ITEM", $params, $types);

						$operationParamsArray = array(
							'CDFILIAL'     => $session['CDFILIAL'],
							'NRVENDAREST'  => $stNrVendaRest,
							'NRCOMANDA'    => $stNrComanda,
							'NRPRODCOMVEN' => $produto['NRPRODCOMVEN']
						);
						$this->KDS->insertKDSOPERACAOTEMP($operationParamsArray, 'cancelItem', $session['NRORG']);

						if (!empty($produto['NRCOMANDAORI']) && !empty($produto['NRPRODCOMORI'])){
							$params = array(
								'TXPRODCOMVEN'   => $motivo['TXPRODCOMVEN'],
								'CDGRPOCOR'      => $motivo['CDGRPOCOR'],
								'CDOCORR'        => $motivo['CDOCORR'],
								'IDSTPRCOMVEN'   => '6',
								'CDSUPERVISOR'   => $dataset['CDSUPERVISOR'],
								'CDFILIAL'       => $session['CDFILIAL'],
								'NRCOMANDA'      => $produto['NRCOMANDAORI'],
								'NRVENDAREST'    => $stNrVendaRest,
								'NRPRODCOMVEN'   => $produto['NRPRODCOMVEN'],
								'IDPRODPRODUZ' 	 => $IDPRODPRODUZ,
								'CDOPERCANC'     => $session['CDOPERADOR'],
								'DTHRPRODCANVEN' => $dataset['DTHRPRODCANVEN']
							);
							$types = array(
								'DTHRPRODCANVEN' => \Doctrine\DBAL\TypeS\Type::DATETIME
							);
							$this->entityManager->getConnection()->executeQuery("SQL_CANCELA_ITEM", $params, $types);
						}
					}
					$params = array($session['CDFILIAL'], $dataset['produto']['nrcomanda'], $dataset['produto']['NRVENDAREST']);
					$acrComVen = $this->entityManager->getConnection()->fetchAll("SQL_ACR_COM_VEN", $params);
					if (!empty($acrComVen)) $acrComVen[0]['VRACRCOMANDA'] -= $acrescimo;
				}*/
				// produto normal (não é promoção inteligente)
				else if (empty($itensCombinados)){

					$params = array(
						'TXPRODCOMVEN'   => $motivo['TXPRODCOMVEN'],
						'CDGRPOCOR'      => $motivo['CDGRPOCOR'],
						'CDOCORR'        => $motivo['CDOCORR'],
						'IDSTPRCOMVEN'   => '6',
						'CDSUPERVISOR'   => $dataset['CDSUPERVISOR'],
						'CDFILIAL'       => $session['CDFILIAL'],
						'NRCOMANDA'      => $stNrComanda,
						'NRVENDAREST'    => $stNrVendaRest,
						'NRPRODCOMVEN'   => $dataset['produto']['NRPRODCOMVEN'],
						'IDPRODPRODUZ'   => $IDPRODPRODUZ,
						'CDOPERCANC'     => $session['CDOPERADOR'],
						'DTHRPRODCANVEN' => $dataset['DTHRPRODCANVEN']
					);
					$types = array(
						'DTHRPRODCANVEN' => \Doctrine\DBAL\TypeS\Type::DATETIME
					);
					$this->entityManager->getConnection()->executeQuery("SQL_CANCELA_ITEM", $params, $types);
					$operationParamsArray = array(
						'CDFILIAL'     => $session['CDFILIAL'],
						'NRVENDAREST'  => $stNrVendaRest,
						'NRCOMANDA'    => $stNrComanda,
						'NRPRODCOMVEN' => $dataset['produto']['NRPRODCOMVEN']
					);
					$this->KDS->insertKDSOPERACAOTEMP($operationParamsArray, 'cancelItem', $session['NRORG']);

					if (!empty($item[0]['NRCOMANDAORI']) && !empty($item[0]['NRPRODCOMORI'])){
						$params = array(
							'TXPRODCOMVEN'   => $motivo['TXPRODCOMVEN'],
							'CDGRPOCOR'      => $motivo['CDGRPOCOR'],
							'CDOCORR'        => $motivo['CDOCORR'],
							'IDSTPRCOMVEN'   => '6',
							'CDSUPERVISOR'   => $dataset['CDSUPERVISOR'],
							'CDFILIAL'       => $session['CDFILIAL'],
							'NRCOMANDA'      => $item[0]['NRCOMANDAORI'],
							'NRVENDAREST'    => $stNrVendaRest,
							'NRPRODCOMVEN'   => $dataset['produto']['NRPRODCOMVEN'],
							'IDPRODPRODUZ'   => $IDPRODPRODUZ,
							'CDOPERCANC'     => $session['CDOPERADOR'],
							'DTHRPRODCANVEN' => $dataset['DTHRPRODCANVEN']
						);
						$types = array(
							'DTHRPRODCANVEN' => \Doctrine\DBAL\TypeS\Type::DATETIME
						);
						$this->entityManager->getConnection()->executeQuery("SQL_CANCELA_ITEM", $params, $types);
					}
				}
				$connection->commit();

			}
			catch(\Exception $e){
				$connection->rollback();
				  throw new \Exception($e->getMessage(), 1);
			}
		}
		else {
            if ($itemSelecionado["IDDIVIDECONTA"] != "S"){
                return array('funcao' => '0', 'error' => '017'); //017 - Quantidade inválida.
            }
        }
		if(!empty($produto['CDPRODPROMOCAO'])&&!empty($itensCombinados) ){
			$arrFilhos=null;
    		$params = array(
        		'CDFILIAL' => $CDFILIAL,
        		'NRVENDAREST' => $produto['NRVENDAREST'],
        		'NRCOMANDA' => $produto['NRCOMANDA'],
        		'NRPRODCOMVEN' => $produto['NRPRODCOMVEN'],
				'CDLOJA' => $session['CDLOJA']
    		);
			//busca itens inseridos somente na itcomandaest
    		$arrFilhos = $this->entityManager->getConnection()->fetchAll("SQL_ITENS_COMBINADOS_FILHOS", $params);
			foreach($arrFilhos as $filho){
				$filho['QTPRODCOMVEN'] = $filho['QTPROCOMEST'];
				array_push($itensCombinados,$filho);
			}
		}

		$ItensImpressao = array();
		$impressoraFiscal = $this->imprimeCancelamentoFiscal($session);

		if (empty($itensCombinados)){ // produto normal
			if ($item[0]['IDIMPCANITEM'] == 'S'){
				foreach ($item as $chave => $produto){
					$impressoraAmbiente = $this->impressaoPedido->buscaImpressoraPorAmbiente($session['CDFILIAL'], $session['CDLOJA'], $produto);

					if(!empty($impressoraFiscal)){
						$impressoras = array ($produto['NRSEQIMPRLOJA'], $produto['NRSEQIMPRLOJA2'], $impressoraFiscal['NRSEQIMPRLOJA']);
					} else {
						$impressoras = array ($produto['NRSEQIMPRLOJA'], $produto['NRSEQIMPRLOJA2']);
					}

					foreach ($impressoras as $key => $value) {
						$allItens[] = array(
							'CDPRODUTO' => $produto['CDPRODUTO'],
							'NMPRODUTO' => $produto['NMPRODUTO'],
							'IDIMPCANITEM' => $produto['IDIMPCANITEM'],
							'NRMESA' => $produto['NRMESA'],
							'DSCOMANDAORI' => $produto['DSCOMANDAORI'],
							'NRPRODCOMORI' => $produto['NRPRODCOMORI'],
							'NRCOMANDAORI' => $produto['NRCOMANDAORI'],
							'VRPRECCOMVEN' => $produto['VRPRECCOMVEN'],
							'QTPRODCOMVEN' => $dataset['produto']['quantidade'],
							'NMSALA' => $stAmbiente,
							'MOTIVO' => $motivo['TXPRODCOMVEN'],
							'CDPORIMPPROD' => $produto['CDPORIMPPROD']
						);

						if ($key == 0) {
							$arrayImpressora = array(
								'NRSEQIMPRPROD' => $produto['NRSEQIMPRPROD'],
								'IDMODEIMPRES' => $produto['IDMODEIMPRES'],
								'DSENDPORTA' => $produto['DSENDPORTA'],
								'NRSEQIMPRLOJA' => $produto['NRSEQIMPRLOJA'],
								'CDPORTAIMPR' => $produto['CDPORTAIMPR'],
								'DSIPIMPR' => $produto['DSIPIMPR'],
								'DSIPPONTE' => $produto['DSIPPONTE'],

								'NRSEQIMPRLOJABK' => $produto['NRSEQIMPPRDBKP'],
								'IDMODEIMPRESBK' => $produto['IDMODEIMPRESBK'],
								'CDPORTAIMPRBK' => $produto['CDPORTAIMPRBK'],
								'DSIPIMPRBK' => $produto['DSIPIMPRBK']
							);
						} else if ($key == 1) {
							$arrayImpressora = array(
								'NRSEQIMPRPROD' => $produto['NRSEQIMPRPROD2'],
								'IDMODEIMPRES' => $produto['IDMODEIMPRES2'],
								'DSENDPORTA' => $produto['DSENDPORTA2'],
								'NRSEQIMPRLOJA' => $produto['NRSEQIMPRLOJA2'],
								'CDPORTAIMPR' => $produto['CDPORTAIMPR2'],
								'DSIPIMPR' => $produto['DSIPIMPR2'],
								'DSIPPONTE' => $produto['DSIPPONTE2'],

								'NRSEQIMPRLOJABK' => $produto['NRSEQIMPPRD2BKP'],
								'IDMODEIMPRESBK' => $produto['IDMODEIMPRESBK2'],
								'CDPORTAIMPRBK' => $produto['CDPORTAIMPRBK2'],
								'DSIPIMPRBK' => $produto['DSIPIMPRBK2']
							);
						}
						if ($key == 2) {
							$arrayImpressora = $impressoraFiscal;
							$arrayImpressora['NRSEQIMPRLOJABK'] = '';
							$arrayImpressora['IDMODEIMPRESBK'] = '';
							$arrayImpressora['CDPORTAIMPRBK'] = '';
							$arrayImpressora['DSIPIMPRBK'] = '';
						}
						if(empty($arrayImpressora)) {
							$arrayImpressora = $impressoraAmbiente;
						}
						array_push($ItensImpressao, array_merge($allItens[0], $arrayImpressora));
					}
				}
			}
		} else { // promoção inteligente
			$c = 0;
			foreach ($itensCombinados as $chave => $produtosCombinados){
				if ($produtosCombinados['IDIMPCANITEM'] == 'S'){
					$impressoraAmbiente = $this->impressaoPedido->buscaImpressoraPorAmbiente($session['CDFILIAL'], $session['CDLOJA'], $produto);

					if(!empty($impressoraFiscal)){
						$impressoras = array ($produtosCombinados['NRSEQIMPRLOJA'], $produtosCombinados['NRSEQIMPRLOJA2'], $impressoraFiscal['NRSEQIMPRLOJA']);
					} else {
						$impressoras = array ($produtosCombinados['NRSEQIMPRLOJA'], $produtosCombinados['NRSEQIMPRLOJA2']);
					}
					$produtosCombinados['MOTIVO'] = $motivo['TXPRODCOMVEN'];
					$produtosCombinados['NRMESA'] = $stNrMesa;

					if (isset($impressoras[0])) {
						$arrayImpressora = null;
						if(empty($impressoras[0])) {
							$arrayImpressora = $impressoraAmbiente;
						} else if(!empty($impressoras[0])) {
							$arrayImpressora = array(
								'NRSEQIMPRPROD' => $produtosCombinados['NRSEQIMPRPROD'],
								'IDMODEIMPRES'  => $produtosCombinados['IDMODEIMPRES'],
								'DSENDPORTA'    => $produtosCombinados['DSENDPORTA'],
								'NRSEQIMPRLOJA' => $produtosCombinados['NRSEQIMPRLOJA'],
								'CDPORTAIMPR'   => $produtosCombinados['CDPORTAIMPR'],
								'DSIPIMPR'      => $produtosCombinados['DSIPIMPR'],
								'DSIPPONTE'     => $produtosCombinados['DSIPPONTE'],

								'NRSEQIMPRLOJABK' => $produtosCombinados['NRSEQIMPPRDBKP'],
								'IDMODEIMPRESBK'  => $produtosCombinados['IDMODEIMPRESBK'],
								'CDPORTAIMPRBK'   => $produtosCombinados['CDPORTAIMPRBK'],
								'DSIPIMPRBK'      => $produtosCombinados['DSIPIMPRBK']
							);
						}
						if(!empty($arrayImpressora)){
							array_push($ItensImpressao, array_merge($produtosCombinados, $arrayImpressora));
						}
					}
					if (isset($impressoras[1])) {
						$arrayImpressora = null;
						if(empty($impressoras[1])) {
							$arrayImpressora = $impressoraAmbiente;
						} else if(!empty($impressoras[1])) {
							$arrayImpressora = array(
								'NRSEQIMPRPROD' => $produtosCombinados['NRSEQIMPRPROD2'],
								'IDMODEIMPRES'  => $produtosCombinados['IDMODEIMPRES2'],
								'DSENDPORTA'    => $produtosCombinados['DSENDPORTA2'],
								'NRSEQIMPRLOJA' => $produtosCombinados['NRSEQIMPRLOJA2'],
								'CDPORTAIMPR'   => $produtosCombinados['CDPORTAIMPR2'],
								'DSIPIMPR'      => $produtosCombinados['DSIPIMPR2'],
								'DSIPPONTE'     => $produtosCombinados['DSIPPONTE2'],

								'NRSEQIMPRLOJABK' => $produtosCombinados['NRSEQIMPPRD2BKP'],
								'IDMODEIMPRESBK'  => $produtosCombinados['IDMODEIMPRESBK2'],
								'CDPORTAIMPRBK'   => $produtosCombinados['CDPORTAIMPRBK2'],
								'DSIPIMPRBK'      => $produtosCombinados['DSIPIMPRBK2']
							);
						}
						if(!empty($arrayImpressora)){
							array_push($ItensImpressao, array_merge($produtosCombinados, $arrayImpressora));
						}
					}
					if (isset($impressoras[2])) {
						$arrayImpressora = null;
						if(empty($impressoras[2])) {
							$arrayImpressora = $impressoraAmbiente;
						} else if(!empty($impressoras[2])) {
							$arrayImpressora = $impressoraFiscal;
							$arrayImpressora['NRSEQIMPRLOJABK'] = '';
							$arrayImpressora['IDMODEIMPRESBK'] = '';
							$arrayImpressora['CDPORTAIMPRBK'] = '';
							$arrayImpressora['DSIPIMPRBK'] = '';
						}
						if(!empty($arrayImpressora)){
							array_push($ItensImpressao, array_merge($produtosCombinados, $arrayImpressora));
						}
					}
				}
			}
		}

		$retornoImpressao = array();
		if (!empty($ItensImpressao)){
			$retornoImpressao = $this->impressaoService->imprimeCancelamento($dataset['chave'], $ItensImpressao, $dataset['mode'], $stDsComanda);
		}

		$this->util->logFOS($session['CDFILIAL'], $session['CDCAIXA'], 'CAN_GEN', $session['CDOPERADOR'], null, "Waiter - Cancelamento de produto", "Cancelamento do produto " . $dataset['produto']['codigo'] . ", referente à mesa " . $mesaOrigem  . ".");

		$result = array('funcao' => '1', 'message' => !empty($retornoImpressao['message']) ? $retornoImpressao['message'] : '');

		if(isset($retornoImpressao['saas'])&&$retornoImpressao['saas']){
			$result = array('funcao' => '1', 'message' => !empty($retornoImpressao['message']) ? $retornoImpressao['message'] : '', 'paramsImpressora' => $retornoImpressao);
		}
		if(isset($retornoImpressao['text'])){
			$text = isset($retornoImpressao['text']) ? $retornoImpressao['text'] : false;
		} else if(isset($retornoImpressao[0]['text'])){
			$text = isset($retornoImpressao[0]['text']) ? $retornoImpressao[0]['text'] : false;
		} else {
			$text = isset($retornoImpressao[0]['text']) ? $retornoImpressao[0]['text'] : false;
		}
		array_push($result, array('text' => $text));

		return $result;
	}

	public function cancelaProdutosDivididos($dataset, $cancel){
		$connection = null;
		try {

            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

			$session       = $this->util->getSessionVars($dataset['chave']);
			$CDFILIAL      = $session['CDFILIAL'];
			$NRVENDAREST   = $dataset["NRVENDAREST"];
			$NRCOMANDA     = $dataset["NRCOMANDA"];
			$selection     = $dataset["selection"];
            $IDPRODPRODUZ  = $dataset["IDPRODPRODUZ"];
			if(empty($NRCOMANDA)){
				throw new \Exception('Nenhum produto foi selecionado.');
			}
            $NRCOMANDA = $NRCOMANDA[0];
            $NRVENDAREST = $NRVENDAREST[0];

			foreach ($selection as $NRPRODORIG){
				$params = array(
					$CDFILIAL,
					$NRVENDAREST,
					$NRCOMANDA,
					$NRPRODORIG
				);
				$values = $this->entityManager->getConnection()->fetchAssoc("SQL_VALIDA_VALORES_ITEM", $params);
				$originalValues = $this->entityManager->getConnection()->fetchAssoc("SQL_VALIDA_VALORES_ORIG", $params);

				if ($values["QTPRODCOMVEN"] != $originalValues["QTPRODCOMVEN"]){
                    return array('funcao' => '0', 'error' => '454'); // 454 - Não foi possivel efetuar o cancelamento. Já houve o pagamento parcial do item.
                }

                $this->entityManager->getConnection()->executeQuery("SQL_DELETA_PRODUTOS_EST", $params);
				$this->entityManager->getConnection()->executeQuery("SQL_DELETA_PRODUTOS", $params);

                $params = array(
                    'CDFILIAL' => $CDFILIAL,
                    'NRVENDAREST' => $NRVENDAREST,
                    'NRCOMANDA' => $NRCOMANDA,
                    'NRPRODCOMVEN' => $NRPRODORIG,
                    'IDPRODPRODUZ' => $IDPRODPRODUZ
                );
                if ($cancel){
                    // Cancela o produto na ITCMDVENORIG.
                    $this->entityManager->getConnection()->executeQuery("SQL_UPDATE_CMD_VEN_ORIG", $params);
                }
                $this->entityManager->getConnection()->executeQuery("SQL_INSERE_ITEM_COMANDA", $params);
                $this->entityManager->getConnection()->executeQuery("SQL_INSERE_ITEM_COMANDAEST", $params);
                $this->entityManager->getConnection()->executeQuery("SQL_DELETA_CMD_EST_ORIG", $params);
                $this->entityManager->getConnection()->executeQuery("SQL_DELETA_CMD_VEN_ORIG", $params);
			}

            $connection->commit();
			return array('funcao' => '1');

		} catch(\Exception $e){
            if ($connection != null) {
                $connection->rollback();
            }
			throw new \Exception ($e->getMessage(),1);
		}

	}

	public function imprimeCancelamentoFiscal($session){ //verifica se é para imprimir na impressora fiscal e retorna os dados
			$params = array(
	            	'CDFILIAL' => $session['CDFILIAL'],
	            	'CDCAIXA' =>$session['CDCAIXA']
	        		);
			$arrayImpressoraFiscal = array();
	        $imprimeFiscal = $this->entityManager->getConnection()->fetchAssoc("IMPRIME_FISCAL_CANCELAMENTO", $params);

	        if($imprimeFiscal['IDIMPCANSATNFCE'] == 'S'){ //parâmetrização para rerificar se imprime ou não
	        	$params = array(
	           		'CDFILIAL' => $session['CDFILIAL'],
	           		'CDCAIXA' =>$session['CDCAIXA'],
	           		'NRORG' => $session['NRORG']
	        	);
	        	$impressoraFiscal = $this->entityManager->getConnection()->fetchAssoc("SQL_BUSCA_DADOS_IMPRESSORA", $params);
	        	$arrayImpressoraFiscal = array( //array com os dados da impresora SAT ou NFCe
						'NRSEQIMPRPROD' => $impressoraFiscal['NRSEQIMPRLOJA'],
						'IDMODEIMPRES' => $impressoraFiscal['IDMODEIMPRES'],
						'NRSEQIMPRLOJA' => $impressoraFiscal['NRSEQIMPRLOJA'],
						'CDPORTAIMPR' => $impressoraFiscal['CDPORTAIMPR'],
						'DSIPIMPR' => $impressoraFiscal['DSIPIMPR'],
						'DSIPPONTE' => $impressoraFiscal['DSIPPONTE']
					);
	        }

	        return $arrayImpressoraFiscal;
	}

}